-- Copyright (c) 2023-2024 Chris Pressey, Cat's Eye Technologies
-- This file is distributed under a BSD license.  See LICENSES directory:
-- SPDX-License-Identifier: LicenseRef-BSD-2-Clause-X-Fountain

module Language.Fountain.Constraint where


data Variable = Var String
    deriving (Show, Ord, Eq)

-- A "constraint expression" is a simple expression appearing inside a constraint.
data ConExpr a = ConVar Variable
               | ConVal a
    deriving (Show, Ord, Eq)

data Constraint a e = Identity
                    | Unify Variable (ConExpr a)
                    | Inc Variable (ConExpr a)
                    | Dec Variable (ConExpr a)
                    | GreaterThan Variable (ConExpr a)
                    | GreaterThanOrEqual Variable (ConExpr a)
                    | LessThan Variable (ConExpr a)
                    | LessThanOrEqual Variable (ConExpr a)
                    | Conj (Constraint a e) (Constraint a e)
                    | Disj (Constraint a e) (Constraint a e)
                    | Not (Constraint a e)
                    | Defined Variable
                    | Lookahead e
                    | IsGenerating
    deriving (Show, Ord, Eq)

depictVar (Var s) = s

depictConExpr (ConVar v) = depictVar v
depictConExpr (ConVal i) = show i

depictConstraint Identity = "T"
depictConstraint (Unify v e) = (depictVar v) ++ " = " ++ (depictConExpr e)
depictConstraint (Inc v e) = (depictVar v) ++ " += " ++ (depictConExpr e)
depictConstraint (Dec v e) = (depictVar v) ++ " -= " ++ (depictConExpr e)
depictConstraint (GreaterThan v e) = (depictVar v) ++ " > " ++ (depictConExpr e)
depictConstraint (GreaterThanOrEqual v e) = (depictVar v) ++ " >= " ++ (depictConExpr e)
depictConstraint (LessThan v e) = (depictVar v) ++ " < " ++ (depictConExpr e)
depictConstraint (LessThanOrEqual v e) = (depictVar v) ++ " <= " ++ (depictConExpr e)
depictConstraint (Conj c1 c2) = (depictConstraint c1) ++ " && " ++ (depictConstraint c2)
depictConstraint (Disj c1 c2) = (depictConstraint c1) ++ " || " ++ (depictConstraint c2)
depictConstraint (Not c) = "~" ++ (depictConstraint c)
depictConstraint (Defined v) = "defined " ++ (depictVar v)
depictConstraint (Lookahead e) = "?" ++ (show e)
depictConstraint (IsGenerating) = "generating"

both Identity x = x
both x Identity = x
both x y = Conj x y
