-- Copyright (c) 2023-2024 Chris Pressey, Cat's Eye Technologies
-- This file is distributed under a BSD license.  See LICENSES directory:
-- SPDX-License-Identifier: LicenseRef-BSD-2-Clause-X-Fountain

module Language.Fountain.Randomness where

import System.Random (uniformR)

--
-- Shuffling lists (for random choices in nondeterminism)
--

pickAt :: [a] -> Int -> (a, [a])
pickAt [] _n = error "empty list"
pickAt (x:xs) n
    | n == 0    = (x, xs)
    | n < 0     = error "range"
    | otherwise =
        let
            (r, rs) = pickAt xs (n-1)
        in
            (r, x:rs)

randomIndex xs prng =
    uniformR (0 :: Int, ((length xs) - 1) :: Int) prng

pickOne xs prng =
    let
        (index, prng') = randomIndex xs prng
        (x, xs') = pickAt xs index
    in
        (x, xs', prng')

shuffle [] prng = ([], prng)
shuffle xs prng =
    let
        (x, xs', prng') = pickOne xs prng
        (xs'', prng'') = shuffle xs' prng'
    in
        (x:xs'', prng'')

