-- Copyright (c) 2023-2024 Chris Pressey, Cat's Eye Technologies
-- This file is distributed under a BSD license.  See LICENSES directory:
-- SPDX-License-Identifier: LicenseRef-BSD-2-Clause-X-Fountain

module Language.Fountain.Store (
    Store, empty, fetch, insert, update, adjust, updateStore, filterStore,
    depictStore
) where

--
-- Variable store.
--

import qualified Data.Map as Map

import Data.List (intercalate)

import Language.Fountain.Constraint


type Store a = Map.Map Variable a


empty = Map.empty
fetch = Map.lookup
insert = Map.insert

update :: (a -> Maybe a) -> Variable -> Store a -> Store a
update = Map.update

adjust :: (a -> a) -> Variable -> Store a -> Store a
adjust = Map.adjust

updateStore :: [Variable] -> [Variable] -> Store a -> Store a -> Store a
updateStore [] [] _sourceStore destStore = destStore
updateStore [] _ _ _ = error "Variable lists must be same length"
updateStore _ [] _ _ = error "Variable lists must be same length"
updateStore (sourceKey:sourceKeys) (destKey:destKeys) sourceStore destStore =
    -- Populate destKey in the new store with the value at sourceKey in the sourceStore
    let
        destStore' = case fetch sourceKey sourceStore of
            Just val -> insert destKey val destStore
            Nothing -> destStore
    in
        updateStore sourceKeys destKeys sourceStore destStore'

filterStore :: Store a -> [Variable] -> Store a
filterStore s vars = Map.filterWithKey (\k _ -> k `elem` vars) s

depictStore s = intercalate "," $ map (\(k, v) -> (depictVar k) ++ "=" ++ (show v)) (Map.toList s)